@@ -75,6 +75,8 @@ class TestPattern(unittest.TestCase):
 
         _('.^010d', dict(type='d', width='10', align='^', fill='.',
             zero=True))
+        _('.2f', dict(type='f', precision='2'))
+        _('10.2f', dict(type='f', width='10', precision='2'))
 
     def test_dot_separated_fields(self):
         # this should just work and provide the named value
@@ -137,6 +139,11 @@ class TestParse(unittest.TestCase):
         r = parse.parse('"{}"?', '"teststr"?')
         self.assertEqual(r[0], 'teststr')
 
+    def test_pipe(self):
+        # issue22: make sure a | in the parse string is handled correctly
+        r = parse.parse('| {}', '| teststr')
+        self.assertEqual(r[0], 'teststr')
+
     def test_fixed(self):
         # pull a fixed value out of string
         r = parse.parse('hello {}', 'hello world')
@@ -164,6 +171,19 @@ class TestParse(unittest.TestCase):
         r = parse.parse('hello {:w} {:w}', 'hello 12 people')
         self.assertEqual(r.fixed, ('12', 'people'))
 
+    def test_precision(self):
+        # pull a float out of a string
+        r = parse.parse('Pi = {:.7f}', 'Pi = 3.1415926')
+        self.assertEqual(r.fixed, (3.1415926, ))
+        r = parse.parse('Pi/10 = {:8.5f}', 'Pi/10 =  0.31415')
+        self.assertEqual(r.fixed, (0.31415, ))
+
+    def test_precision_fail(self):
+        # floats must have a leading zero
+        # IS THIS CORRECT?
+        r = parse.parse('Pi/10 = {:8.5f}', 'Pi/10 = .31415')
+        self.assertEqual(r, None)
+
     def test_custom_type(self):
         # use a custom type
         r = parse.parse('{:shouty} {:spam}', 'hello world',
@@ -205,7 +225,7 @@ class TestParse(unittest.TestCase):
         r = parse.parse('{n:d} {n:d}', '1 2')
         self.assertEqual(r, None)
 
-    def test_named_repeated_type_fail_value(self):
+    def test_named_repeated_type_mismatch(self):
         # test repeated name with mismatched type
         self.assertRaises(parse.RepeatedNameError, parse.compile,
             '{n:d} {n:w}')
@@ -454,6 +474,11 @@ class TestParse(unittest.TestCase):
         y('a {:ta} b', 'a November-21-2011 10:21:36 AM +1000 b', aest_d)
         y('a {:ta} b', 'a November-21-2011 b', d)
 
+        # ts   Linux System log format        datetime
+        y('a {:ts} b', 'a Nov 21 10:21:36 b',  datetime(datetime.today().year, 11, 21, 10, 21, 36))
+        y('a {:ts} b', 'a Nov  1 10:21:36 b',  datetime(datetime.today().year, 11, 1, 10, 21, 36))
+        y('a {:ts} b', 'a Nov  1 03:21:36 b',  datetime(datetime.today().year, 11, 1, 3, 21, 36))
+
         # th   HTTP log format date/time                   datetime
         y('a {:th} b', 'a 21/Nov/2011:10:21:36 +1000 b', aest_d)
         y('a {:th} b', 'a 21/Nov/2011:10:21:36 +10:00 b', aest_d)
